# Capture SDK Flutter
This is the Flutter SDK for Socket Mobile's Capture library. 

## Getting Started
Before installing the package, you will need to adjust a few properties in your Flutter app.

# iOS
You will need to update the podfile in the `ios` directory of your app in order to be compatible with the version used in our SDK. 
```
project_root/ios/Podfile
# Uncomment this line to define a global platform for your project
# platform :ios, '9.0'
```

Uncomment out the second line and change the version from `'9.0'` to `'10.1'`.

Next, go to `ios/Runner/Info.plist` and at the bottom, just above `</dict>`, include the below code.

```
<key>NSBluetoothAlwaysUsageDescription</key>
	<string>Bluetooth is needed to connect to a Socket Mobile device</string>
	<key>UISupportedExternalAccessoryProtocols</key>
	<array>
		<string>com.socketmobile.chs</string>
	</array>
```

If you do not do this, you will encounter an error saying `This app has crashed because it attempted to access privacy-sensitive data without a usage description.  The app's Info.plist must contain an NSBluetoothAlwaysUsageDescription key with a string value explaining to the user how the app uses this data.`. This is a bluetooth security measure, and to permit bluetooth access you will need to add the above entry in your `Info.plist` file. 

# Android
THIS IS WHERE WE WILL ADD WHAT YOU NEED TO INCLUDE FOR ANDROID USAGE.

## Installation
 
Install the flutter package using `flutter pub get capture_flutter_beta`. It will add the below line to your `pubspec.yaml` file.

```
dependencies:
  flutter:
    capture_flutter_beta: 1.2.54   
```

In the `main.dart`, you can import the capture flutter sdk by adding this line to the top of your file. 

```
import 'package:capture_flutter_beta/capture_flutter_beta.dart';
```

Create a `Capture` instance with the line `Capture capture = Capture(logger);`. `logger` is an optional argument that is passed to `Capture` and can be helpful with tracing values and requests throughout the app, particularly in `Capture` and `HttpTransport` where the bulk of the Capture logic and requests are handled. 

Open the connection wth the Capture library by using the `open` method on the `Capture` instance. See below.

```
int? response = await capture.openClient(appInfo, _onCaptureEvent);
stat = 'handle: $response';
mess = 'capture open success';
```

`appInfo` is an instance of the `AppInfo` class and consists of the same parameters found in other Capture SDKs. See an example of `appInfo` below.

```
final appInfo = AppInfo(
        'android:com.example.example',
        'MC4CFQDNCtjazxILEh8oyT6w/wlaVKqS1gIVAKTz2W6TB9EgmjS1buy0A+3j7nX4',
        'ios:com.example.example',
        'MC0CFA1nzK67TLNmSw/QKFUIiedulUUcAhUAzT6EOvRwiZT+h4qyjEZo9oc0ONM=',
        'bb57d8e1-f911-47ba-b510-693be162686a');
```

To generate app info, head to the [docs](https://www.socketmobile.com/developers/portal/application-details/appkey-registration) and follow the prompts to register your app and generate your `appInfo` credentials.

Next, `_onCaptureEvent` is the callback passed to `open` that can handle the event notifications from the Capture SDK. Below are three important events to consider, which are accessible in the `CaptureEventIds` class. 

`deviceArrival` is the event that is triggered when the scanner connects to your device. 

`deviceRemoval` is the event that is triggered when the scanner is disconnected from the device.

`decodedData` is the event that is triggered when you scan something with the connected scanner.

See an example of event handling below in `_onCaptureEvent`.

```
_onCaptureEvent(e, handle) {

    if (e == null) {
      return;
    } else if (e.runtimeType == CaptureException) {
      _updateVals("${e.code}", e.message, e.method, e.details);
      return;
    }

    logger.log('onCaptureEvent from: ', '$handle');

    switch (e.id) {
      case CaptureEventIds.deviceArrival:
        Capture deviceCapture = Capture(logger);

        setState(() {
          _deviceCapture = deviceCapture;
        });

        _openDeviceHelper(deviceCapture, e);
        break;
      case CaptureEventIds.deviceRemoval:
        _closeDeviceHelper(e, handle);
        break;

      case CaptureEventIds.decodedData:
        setState(() {
          /// storing scanned data in state for future use
          _currentScan = e;
        });
        _updateVals('Decoded Data', "Successful scan!");
        break;
    }
  }
```

The data the user will need to anticipate will be a `CaptureEvent` which might contain an instance of `DecodedData`, `CaptureException`, a client or device handle, etc. When you first connect to the service, the response will be a client handle (an integer) or a `CaptureException` instance will be thrown.

It's important to create another `Capture` instance when you have successfully connected the scanner to your device (see `var newCapture = Capture(logger);`). This capture instance is tied to your device and will allow the root capture instance to remain open, regardless of what happens with your device. The new instance allows you to create a capture connection to the device to handle various actions specific to the connected device, such as `getProperty` and `setProperty`. 

`getProperty` enables you to retrieve specific values for the connected device, such as `friendlyNameDevice` which is the property corresponding to the scanners given name (as opposed to id/guid). Create a property instance with the provided values corresponding to the property id and type, as well an empty object. You can retrieve the friendly name of a device with the following request. 

```
void _handleGetNameProperty(){
   
    var property = CaptureProperty(
        CapturePropertyIds().friendlyNameDevice, 
        CapturePropertyTypes().none,
        {}
    );

    _newCapture.getProperty(property).then((response){
        if (response['error'] != null){
            print(response['error']['code']);
        } else {
            print('response['value']);
        }
    });
}
```

`setProperty` allows you to update the value of a specific property. The `CaptureProperty` instance you provide is similar to the one in `getProperty` but instead of an empty object for the value, you will send the value you want assigned to the property. You will also send the data type for the property as well instead of `CapturePropertyTypes().none`. In the case of `friendlyNameDevice`, you would send a string value with the type `CapturePropertyTypes().string`. See below.

```
void _handleSetNameProperty(){

    var property = CaptureProperty(
        CapturePropertyIds().friendlyNameDevice, 
        CapturePropertyTypes().string, 
        _newName
    );
    

    _newCapture.setProperty(property).then((res){
       
        if (res['error'] != null){
            print(response['error']['code']);
        } else {
            print('successfully updated device friendly name!')
        }
        
    });
}
```

The response in `setProperty` does not contain a `value` property. You can access the updated value by calling the `getProperty` request after a successful `setProperty` call or you can use the locally stored value that you're using for assignment. 

## Important
We don’t yet have a page in the dev portal to register for a Flutter app. For now, you need to register using Android and iOS registration or React Native and iOS registration. Your appInfo should look like the below in your app. Below is an example of credentials generated during iOS registration.

![iOS app registration](readme.png)

In order for your flutter app to work, you will need to add two more properties to your `AppInfo` instance. 

```
AppInfo(
        'web:com.socketmobile.reactjs.native.example.example',
        'MCwCFE6Cin/6r1kVHNHN8816Rowi8sPMAhQzruU80/2T/1ihJBQ1rqvCPodMlA==',
        'ios:com.example.betaSdkTest',
        'MCwCFE6Cin/6r1kVHNHN8816Rowi8sPMAhQzruU80/2T/1ihJBQ1rqvCPodMlA==',
        'ecc6c526-970b-ec11-b6e6-0022480a2304');
```

In order for it to work, you will need to add five argments to `AppInfo` in the following order:

  1. React Native or Android AppId
  2. AppKey after iOS registration
  3. iOS AppID
  4. AppKey after iOS registration
  5. Developer ID

In the near future we will have a separate flutter registration option that would remove the need to register your app with multiple platforms.